const express = require("express");
const router = express.Router();
const DecryptWithServerKey = require("./../../services/decryptionservice");
const UserSchema = require("../../model/patient/patient");
const StellarSDK = require("stellar-sdk");
const PatientTransaction = require("../../model/stellar/transactionDetails");
const EncryptionService = require("./../../services/encryptionservice");

//Add user emotions to blockchain and transaction details to database
router.post("/", async (req, res) => {
  // const userPubKey = DecryptWithServerKey(req.body.pubkey);
  // const userSeed = DecryptWithServerKey(req.body.seed);
  const userPubKey = req.body.pubkey;
  const userSeed = req.body.seed;
  const emotion = req.body.emotion;

  //get user details from the database
  try {
    let patientDetails = await UserSchema.findOne({
      publickey: userPubKey,
    });
    if (patientDetails == "") {
      console.log(
        "No patient details found for the public key : " + userPubKey
      );
      console.log("/patient/emotion - adding user emotion failed");
      return res.status(400).json({
        message: "No patient details found for the public key : " + userPubKey,
      });
    } else {
      //get the previous hash
      let previousHash = patientDetails.currenthash;

      //load user account
      const userSourceKeyPair = StellarSDK.Keypair.fromSecret(userSeed);
      const userPublicKey = userSourceKeyPair.publicKey();
      const server = new StellarSDK.Server(process.env.STELLARTESTNET);
      const account = await server.loadAccount(userPublicKey);
      const fee = await server.fetchBaseFee();
      const timestamp = new Date().toString();
      //create transaction
      const transaction = new StellarSDK.TransactionBuilder(account, {
        fee,
        networkPassphrase: StellarSDK.Networks.TESTNET,
      })
        .addMemo(StellarSDK.Memo.text("Emotion record"))
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Previous",
            value: previousHash,
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Emotion",
            value: EncryptionService.EncryptWithStellarSeed(emotion, userSeed),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Timestamp",
            value: timestamp,
          })
        )
        .setTimeout(30)
        .build();

      //sign transaction
      transaction.sign(userSourceKeyPair);

      //send the transaction to blockchain
      try {
        const transactionResult = await server.submitTransaction(transaction);
        console.log(
          "Emotion transaction sent to blockchain : HASH : " +
            transactionResult.hash
        );
        console.log(
          "View emotion transaction at : https://horizon-testnet.stellar.org/transactions/" +
            transactionResult.hash
        );
        try {
          //update the patient collection
          const updatePatient = await UserSchema.updateOne(
            {
              publickey: userPubKey,
            },
            { currenthash: transactionResult.hash }
          );
          if (!updatePatient) {
            console.log("Failed to update the hash : " + updatePatient);
            console.log("/patient/emotion - adding user emotion failed");
            return res.status(500).json({
              message: "Failed to update the hash : " + updatePatient,
            });
          } else {
            //create insert object
            const patientTransactionObj = new PatientTransaction({
              transactionhash: transactionResult.hash,
              timestamp: new Date(),
              publickey: userPubKey,
            });

            try {
              //insert transaction details into the database
              const saveTransaction = await patientTransactionObj.save();
              console.log(
                "Emotion transaction saved to database : " + saveTransaction
              );
              res.status(200).json({
                message: "Emotion details added to the blockchain",
                hash: transactionResult.hash,
              });
              console.log("/patient/emotion - adding user emotion success");
            } catch (error) {
              console.log(
                "Error when adding transaction details to DB: " + error
              );
              res.status(500).json({
                message:
                  "Error when adding transaction details to DB: " + error,
              });
              console.log("/patient/emotion - adding user emotion failed");
            }
          }
        } catch (error) {
          console.log("Error updating the hash : " + error);
          res.status(500).json({
            message: "Error updating the hash : " + error,
          });
          console.log("/patient/emotion - adding user emotion failed");
        }
      } catch (error) {
        console.log("Error when sending transaction to blockchain : " + error);
        res.status(500).json({
          message: "Error when sending transaction to blockchain : " + error,
        });
        console.log("/patient/emotion - adding user emotion failed");
      }
    }
  } catch (error) {
    console.log(
      "Error when searching for patient details in the server DB : " +
        error.message
    );
    res.status(404).json({
      message:
        "Error when searching for patient details in the server DB : " + error,
    });
    console.log("/patient/emotion - adding user emotion failed");
  }
});
module.exports = router;
