import { useNavigation } from "@react-navigation/core";
import {
  View,
  TextInput,
  Button,
  StyleSheet,
  Alert,
  ActivityIndicator,
} from "react-native";
import React, { useState } from "react";
import EncryptWithServerKey from "../../services/encryptByServerKey";
import { LOCALBACKEND } from "../../env";
import DecryptWithServerKey from "../../services/decryptByServerKey";
import AsyncStorage from "@react-native-async-storage/async-storage";

export default function PractitionerRegistration() {
  const navigation = useNavigation();
  const [username, setUsername] = useState("");
  const [password, setPassword] = useState("");
  const [email, setEmail] = useState("");
  const [fullName, setFullName] = useState("");
  const [workspace, setWorkspace] = useState("");
  const [isLoading, setIsLoading] = useState(false);

  const handleNext = async () => {
    setIsLoading(true);

    //encrypt the data
    const encryptedUsername = EncryptWithServerKey(username);
    const encryptedPassword = EncryptWithServerKey(password);
    const encryptedEmail = EncryptWithServerKey(email);
    const encryptedFullName = EncryptWithServerKey(fullName);
    const encryptedWorkplace = EncryptWithServerKey(workspace);

    //create JSON object
    const registerObject = {
      username: encryptedUsername,
      password: encryptedPassword,
      email: encryptedEmail,
      fullName: encryptedFullName,
      workspace: encryptedWorkplace,
    };

    const registerUrl = LOCALBACKEND + `/practitioner/register`;

    try {
      const regResponse = await fetch(registerUrl, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify(registerObject),
      });

      const responseData = await regResponse.json();
      const { message, hash, pubkey, seed, rsapk, rsask } = responseData;
      //Decrypt the data
      const decryptedStellarPublicKey = DecryptWithServerKey(pubkey);
      const decryptedStellarSecretKey = DecryptWithServerKey(seed);
      const decryptedRsaPublicKey = DecryptWithServerKey(rsapk);
      const decryptedRsaSecretKey = DecryptWithServerKey(rsask);

      //save in the async storage
      try {
        await AsyncStorage.setItem(
          "practitionerStellarPublicKey",
          decryptedStellarPublicKey
        );
        await AsyncStorage.setItem(
          "practitionerStellarSecretKey",
          decryptedStellarSecretKey
        );
        await AsyncStorage.setItem(
          "practitionerRsaPublicKey",
          decryptedRsaPublicKey
        );
        await AsyncStorage.setItem(
          "practitionerRsaSecretKey",
          decryptedRsaSecretKey
        );

        const alertMessage =
          `Public key : ` +
          decryptedStellarPublicKey +
          `\nSecret key : ` +
          decryptedStellarSecretKey +
          `\n Save the keys in a safe place`;
        setIsLoading(false);

        Alert.alert("Account keys", alertMessage, [
          {
            text: "Copy",
            onPress: () => {},
          },
          {
            text: "Close",
            style: "cancel",
            onPress: () => {
              navigation.navigate("PractitionerMainView");
            },
          },
        ]);
      } catch (errorWhenStoringInTheAsyncStorage) {
        console.log(
          "Error when saving details in the async storage : ",
          errorWhenStoringInTheAsyncStorage
        );
        setIsLoading(false);
      }
    } catch (errorWhenRegisteringPractitioner) {
      console.log(
        "Error when registering practitioner : ",
        errorWhenRegisteringPractitioner
      );
      setIsLoading(false);
    }
  };

  return (
    <View style={styles.container}>
      {isLoading ? (
        <ActivityIndicator size="large" color="#560CCE" />
      ) : (
        <>
          <TextInput
            style={styles.input}
            placeholder="User name"
            value={username}
            onChangeText={(text) => setUsername(text)}
          />
          <TextInput
            style={styles.input}
            placeholder="Password"
            value={password}
            onChangeText={(text) => setPassword(text)}
            secureTextEntry={true}
          />
          <TextInput
            style={styles.input}
            placeholder="Full Name"
            value={fullName}
            onChangeText={(text) => setFullName(text)}
          />
          <TextInput
            style={styles.input}
            placeholder="Workspace"
            value={workspace}
            onChangeText={(text) => setWorkspace(text)}
          />
          <TextInput
            style={styles.input}
            placeholder="Email"
            value={email}
            onChangeText={(text) => setEmail(text)}
          />
          <Button title="Register" onPress={handleNext} />
        </>
      )}
    </View>
  );
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    alignItems: "center",
    justifyContent: "center",
  },
  input: {
    width: "80%",
    height: 40,
    marginVertical: 10,
    paddingHorizontal: 10,
    borderColor: "gray",
    borderWidth: 1,
    borderRadius: 5,
  },
});
